@@ -13,9 +13,10 @@
 #' @param fromID character vector same length as fromPlace
 #' @param toID character vector same length as toPlace
 #' @param mode character vector of one or more modes of travel valid values
-#'   TRANSIT, WALK, BICYCLE, CAR, BUS, RAIL, SUBWAY, TRAM, FERRY, BICYCLE_RENT,
-#'   BICYCLE_PARK, CAR_PARK, CABLE_CAR, GONDOLA, FUNICULAR, AIRPLANE, default
-#'   CAR. Not all combinations are valid e.g. c("WALK","BUS") is valid but
+#'   "TRANSIT","BUS", "RAIL", "SUBWAY","TRAM", "FERRY", "GONDOLA", "FUNICULAR",
+#'   "AIRPLANE", "CABLE_CAR", "WALK", "BICYCLE", "BICYCLE_RENT", "BICYCLE_PARK",
+#'   "CAR", "CAR_PARK", "CAR_HAIL", "CARPOOL", "CAR_DROPOFF", "CAR_PICKUP",
+#'   default "CAR". Not all combinations are valid e.g. c("WALK","BUS") is valid but
 #'   c("WALK","CAR") is not.
 #' @param date_time POSIXct, a date and time, defaults to current date and time
 #' @param arriveBy Logical, Whether the trip should depart or arrive at the
@@ -128,11 +129,12 @@ otp_plan <- function(otpcon = NA,
   mode <- toupper(mode)
   checkmate::assert_subset(mode,
                            choices = c(
-                             "TRANSIT", "WALK", "BICYCLE",
-                             "CAR", "BUS", "RAIL", "SUBWAY",
-                             "TRAM", "FERRY","BICYCLE_RENT",
-                             "BICYCLE_PARK","CAR_PARK","CABLE_CAR",
-                             "GONDOLA","FUNICULAR","AIRPLANE"
+                             "TRANSIT","BUS", "RAIL", "SUBWAY","TRAM", "FERRY",
+                             "GONDOLA","FUNICULAR","AIRPLANE","CABLE_CAR",
+                             "WALK",
+                             "BICYCLE","BICYCLE_RENT","BICYCLE_PARK",
+                             "CAR","CAR_PARK",
+                             "CAR_HAIL","CARPOOL","CAR_DROPOFF","CAR_PICKUP"
                            ),
                            empty.ok = FALSE
   )
@@ -250,22 +252,22 @@ otp_plan <- function(otpcon = NA,
     query <- c(query, routeOptions)
   }
 
-  if(!is.null(fromID)){
-    fromID <- data.table::data.table(fromID = fromID,
-                                     fromPlace = gsub("%2C",",",fromPlace))
-    fromID <- unique(fromID)
-    if(any(duplicated(fromID$fromID))){
-      stop("Can't have two fromIDs with the same location, coordinates are rounded to 9 dp")
-    }
-  }
-  if(!is.null(toID)){
-    toID <- data.table::data.table(toID = toID,
-                                     toPlace = gsub("%2C",",",toPlace))
-    toID <- unique(toID)
-    if(any(duplicated(toID$toID))){
-      stop("Can't have two toIDs with the same location, coordinates are rounded to 9 dp")
-    }
-  }
+  # if(!is.null(fromID)){
+  #   fromID <- data.table::data.table(fromID = fromID,
+  #                                    fromPlace = gsub("%2C",",",fromPlace))
+  #   fromID <- unique(fromID)
+  #   if(any(duplicated(fromID$fromID))){
+  #     stop("Can't have two fromIDs with the same location, coordinates are rounded to 9 dp")
+  #   }
+  # }
+  # if(!is.null(toID)){
+  #   toID <- data.table::data.table(toID = toID,
+  #                                    toPlace = gsub("%2C",",",toPlace))
+  #   toID <- unique(toID)
+  #   if(any(duplicated(toID$toID))){
+  #     stop("Can't have two toIDs with the same location, coordinates are rounded to 9 dp")
+  #   }
+  # }
 
   # Send Requests
   urls <- build_urls(routerUrl,fromPlace, toPlace, query)
@@ -273,21 +275,48 @@ otp_plan <- function(otpcon = NA,
   progressr::handlers("cli")
   results <- progressr::with_progress(otp_async(urls, ncores))
 
-  if(length(results) == 0){
+
+  message(Sys.time()," processing results")
+  results_routes <- RcppSimdJson::fparse(results,
+                                         query = "/plan/itineraries",
+                                         query_error_ok = TRUE,
+                                         parse_error_ok = TRUE,
+                                         always_list = TRUE)
+
+
+  if(is.null(fromID)){
+    fromID <- unlist(RcppSimdJson::fparse(results,
+                                          query = "/requestParameters/fromPlace",
+                                          query_error_ok = TRUE,
+                                          parse_error_ok = TRUE), use.names = FALSE)
+  }
+
+  if(is.null(toID)){
+    toID <- unlist(RcppSimdJson::fparse(results,
+                                        query = "/requestParameters/fromPlace",
+                                        parse_error_ok = TRUE,
+                                        query_error_ok = TRUE), use.names = FALSE)
+
+  }
+
+
+
+
+  results_errors <- RcppSimdJson::fparse(results, query = "/error",
+                                         query_error_ok = TRUE,
+                                         parse_error_ok = TRUE,
+                                         always_list = TRUE)
+  results_errors <- results_errors[lengths(results_errors) > 0]
+
+
+  if(sum(lengths(results_routes)) == 0){
     stop("No results returned, check your connection")
   }
 
-  message(Sys.time()," processing results")
-  results_routes <- RcppSimdJson::fparse(results, query = "/plan/itineraries", query_error_ok = TRUE, always_list = TRUE)
-  fp <- unlist(RcppSimdJson::fparse(results, query = "/requestParameters/fromPlace", query_error_ok = TRUE), use.names = FALSE)
-  tp <- unlist(RcppSimdJson::fparse(results, query = "/requestParameters/toPlace", query_error_ok = TRUE), use.names = FALSE)
-
-  results_errors <- RcppSimdJson::fparse(results, query = "/error", query_error_ok = TRUE, always_list = TRUE)
-  results_errors <- results_errors[lengths(results_errors) > 0]
-
   results_routes <- purrr::pmap(.l = list(itineraries = results_routes,
-                                          fp = fp,
-                                          tp = tp),
+                                          fp = fromID,
+                                          tp = toID
+                                          ),
                                 .f = otp_json2sf,
                                 full_elevation = full_elevation,
                                 get_geometry = get_geometry,
@@ -296,7 +325,7 @@ otp_plan <- function(otpcon = NA,
                                 .progress = TRUE)
 
 
-  results_routes <- data.table::rbindlist(results_routes, fill = TRUE)
+  results_routes <- data.table::rbindlist(results_routes, fill = TRUE, use.names=TRUE)
 
   origin <- .POSIXct(0, tz = "GMT")
   results_routes$startTime <- as.POSIXct.numeric(results_routes$startTime / 1000,
@@ -319,12 +348,12 @@ otp_plan <- function(otpcon = NA,
   results_routes <- results_routes[seq_len(nrow(results_routes)), ]
   results_routes <- as.data.frame(results_routes)
 
-  if(!is.null(fromID)){
-    results_routes$fromPlace <- fromID$fromID[match(results_routes$fromPlace, fromID$fromPlace)]
-  }
-  if(!is.null(toID)){
-    results_routes$toPlace <- toID$toID[match(results_routes$toPlace, toID$toPlace)]
-  }
+  # if(!is.null(fromID)){
+  #   results_routes$fromPlace <- fromID$fromID[match(results_routes$fromPlace, fromID$fromPlace)]
+  # }
+  # if(!is.null(toID)){
+  #   results_routes$toPlace <- toID$toID[match(results_routes$toPlace, toID$toPlace)]
+  # }
 
   if(get_geometry){
     results_routes$geometry <- sf::st_as_sfc(results_routes$geometry, crs = 4326)
@@ -471,24 +500,11 @@ build_urls <- function (routerUrl,fromPlace, toPlace, query){
 #' @noRd
 otp_async <- function(urls, ncores, iso_mode = FALSE, post = FALSE){
 
-
-  # Success Function
-  otp_success <- function(res){
-    p()
-    data <<- c(data, rawToChar(res$content))
-  }
-  # Fail Function
-  otp_failure <- function(msg){
-    p()
-    cat("Error: ", msg, "\n")
-  }
-
   t1 <- Sys.time()
-
+  p <- progressr::progressor(length(urls))
+  out <- vector('list', length(urls))
   pool <- curl::new_pool(host_con = ncores)
-  data <- list()
-
-  for(i in seq_len(length(urls))){
+  lapply( seq_along(urls), function(i){
     h <- curl::new_handle()
     if(post){
       curl::handle_setopt(h, post = TRUE)
@@ -496,17 +512,25 @@ otp_async <- function(urls, ncores, iso_mode = FALSE, post = FALSE){
     if(iso_mode){
       h <- curl::handle_setheaders(h, "Accept" = "application/json")
     }
+    success <- function(res){
+      p()
+      out[[i]] <<- rawToChar(res$content)
+    }
+    failure <- function(res){
+      p()
+      cat("Error: ", res, "\n")
+      out[[i]] <<- paste0("Error: ", res)
+    }
     curl::curl_fetch_multi(urls[i],
-                           otp_success,
-                           otp_failure ,
+                           done = success,
+                           fail = failure,
                            pool = pool,
                            handle = h)
-  }
-  p <- progressr::progressor(length(urls))
+  })
   curl::multi_run(timeout = Inf, pool = pool)
   t2 <- Sys.time()
   message("Done in ",round(difftime(t2,t1, units = "mins"),1)," mins")
-  return(unlist(data, use.names = FALSE))
+  return(unlist(out, use.names = FALSE))
 }
 
 #' Convert output from OpenTripPlanner into sf object
@@ -538,7 +562,7 @@ otp_json2sf <- function(itineraries, fp, tp,
 
   names(legs) <- seq_len(length(legs))
   legs <- legs[!is.na(legs)]
-  legs <- data.table::rbindlist(legs, fill = TRUE, idcol = "route_option")
+  legs <- data.table::rbindlist(legs, fill = TRUE, idcol = "route_option", use.names=TRUE)
   names(legs) <- paste0("leg_",names(legs))
   names(legs)[names(legs) == "leg_route_option"] <- "route_option"
   names(legs)[names(legs) == "leg_geometry"] <- "geometry"
