@@ -196,13 +196,13 @@ test_that("otp_isochrone input validation", {
   )
   skip_on_cran()
   expect_error(otp_isochrone(otpcon, fromPlace = c(1, 1)),
-    regexp = "No results returned, check your connection"
+    regexp = "No results returned, check your connection|Failed to connect to localhost"
   )
 })
 
 context("Test config functions")
 
-test_that("otp_make_config tests", {
+test_that("otp_make_config tests v1", {
   skip_on_cran()
   config_router <- otp_make_config("router")
   config_build <- otp_make_config("build")
@@ -234,6 +234,38 @@ test_that("otp_make_config tests", {
   )))
 })
 
+test_that("otp_make_config tests v2", {
+  skip_on_cran()
+  config_router <- otp_make_config("router", version = 2)
+  config_build <- otp_make_config("build", version = 2)
+  config_otp <- otp_make_config("otp", version = 2)
+
+  expect_is(config_router, "list")
+  expect_is(config_build, "list")
+  expect_is(config_otp, "list")
+
+  expect_true(otp_validate_config(config_router, version = 2))
+  expect_true(otp_validate_config(config_build, version = 2))
+  expect_true(otp_validate_config(config_otp, version = 2))
+
+  #dir.create(file.path(tempdir(), "otptests", "graphs"))
+  dir.create(file.path(tempdir(), "otptests", "graphs", "configtests2"))
+
+  otp_write_config(config_router, dir = file.path(tempdir(), "otptests"), router = "configtests2")
+  otp_write_config(config_build, dir = file.path(tempdir(), "otptests"), router = "configtests2")
+  otp_write_config(config_otp, dir = file.path(tempdir(), "otptests"), router = "configtests2")
+
+  expect_true(file.exists(file.path(
+    tempdir(), "otptests", "graphs", "configtests2", "router-config.json"
+  )))
+  expect_true(file.exists(file.path(
+    tempdir(), "otptests", "graphs", "configtests2", "build-config.json"
+  )))
+  expect_true(file.exists(file.path(
+    tempdir(), "otptests", "graphs", "configtests2", "otp-config.json"
+  )))
+})
+
 context("Test graph checking")
 
 test_that("otp_build_graph input validation", {
